#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <cvirte.h>     /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include "rsupl.h"
#include "rsupl_loopp.h"

static int configHdl, panelHdl;
static ViSession instrSession;
static ViStatus status;

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)   /* Needed if linking in external compiler; harmless otherwise */
        return -1;  /* out of memory */
    if ((configHdl = LoadPanel (0, "rsupl_loopp.uir", CONFIG)) < 0)
        return -1;
    if ((panelHdl = LoadPanel (0, "rsupl_loopp.uir", PANEL)) < 0)
        return -1;
    DisplayPanel (panelHdl);
    InstallPopup (configHdl);
    RunUserInterface ();
    return 0;
}

void errorQuit(char* func, ViStatus status) {
    static char errMsg[512];
    static char label[256];
    if (func != 0) {
        sprintf (label, "%s ERROR", func);
    } else {
        strcpy (label, "ERROR");
    }
    rsupl_error_message (instrSession, status, errMsg);
    MessagePopup (label, errMsg);
    exit (0);
}

int CVICALLBACK okCall (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    char strBuf[256], strBuf2[256];
    int reset, audioMonitor, i;
    ViReal64 value;

    switch (event)
        {
        case EVENT_COMMIT:
            // initialize and reset to a known state
            GetCtrlVal (configHdl, CONFIG_INSTRDESCR, strBuf);
            GetCtrlVal (configHdl, CONFIG_RESET, &reset);
            if ((status = rsupl_init (strBuf, VI_TRUE, (ViBoolean)reset, &instrSession)) < 0) {
                errorQuit("rsupl_init", status);
            }

            // config trigger
            if ((status = rsupl_confInstrTrig (instrSession, VI_FALSE)) < 0) {
                errorQuit("rsupl_confInstrTrig", status);
            }
            
            // config universal analog generator
            if ((status = rsupl_confGenAnStp (instrSession, 3, VI_FALSE, 0, 10.0, 1000.0,
                                        1.0, 0, 0)) < 0) {
                errorQuit("rsupl_confGenAnStp", status);
            }
            
            // config sine output
            if ((status = rsupl_confGenFuncSin (instrSession, VI_FALSE, 1000.0, 0, VI_FALSE, 0.0, 0, 1.0, 0,
                                       VI_FALSE, "*.veq", VI_FALSE, 0.0, 15, 0, VI_FALSE)) < 0) {
                errorQuit("rsupl_confAnalyzerAnalStp", status);
            }
            
            // config 22kHz analog analyzer
            if ((status = rsupl_confAnalyzerAnalStp (instrSession, 600.0, 0, VI_TRUE, VI_FALSE, 1,
                                                     1, VI_FALSE, VI_FALSE, 1, 0, 8, 0, VI_FALSE)) < 0) {
                errorQuit("rsupl_confAnalyzerAnalStp", status);
            }
            
            // config RMS measurement
            if ((rsupl_confAnalyzerFuncRMSSN (instrSession, VI_FALSE, 2, 0.3, 0, 0, 0.0, VI_FALSE, 0,
                                              1.0, 0, 0, 0, 10.0)) < 0) {
                errorQuit("rsupl_confAnalyzerFuncFFT", status);
            }
            
            // select filter
            if ((status = rsupl_confAnalyzerFilterSel (instrSession, 10, 0, 0)) < 0) {
                errorQuit("rsupl_confAnalyzerFuncFFT", status);
            }
            
            // enhance measurement speed
            // switch off display & cursors
            if ((status = rsupl_confInstrDisplay (instrSession,VI_FALSE, VI_FALSE)) < 0) {
                errorQuit("rsupl_confInstrDisplay", status);
            }
            // switch off sense 2 & 3
            if ((status = rsupl_confInstrSense (instrSession, VI_TRUE, VI_FALSE, VI_FALSE)) < 0) {
                errorQuit("rsupl_confInstrDisplay", status);
            }
            
            // audio monitor available?
            if ((status = rsupl_confInstrOpt (instrSession, strBuf2, strBuf2, strBuf2,
                                              strBuf2, strBuf, strBuf2, strBuf2)) < 0) {
                errorQuit("rsupl_confInstrDisplay", status);
            }
            audioMonitor = !strstr (strBuf, "-NA-");
            
            if (audioMonitor) {
                // config audio monitor
                ViReal64 volume;
                GetCtrlVal (configHdl, CONFIG_VOLUME, &volume);
                if ((status = rsupl_confInstrAudioMonitor (instrSession, VI_TRUE, 1, VI_FALSE, 0.0,
                                             volume)) < 0) {
                    errorQuit("rsupl_confInstrDisplay", status);
                }
            }
            
            // For Loop controlling frequency sweep
            for (i = 0; i < 100; i++) {
                if ((status = rsupl_setFreqLevel (instrSession, VI_TRUE, pow (10.0, i/50.0+2.0), 0, VI_FALSE, 1.0,
                                             0)) < 0) {
                    errorQuit("rsupl_confInstrDisplay", status);
                }
                if ((status = rsupl_sendTrig (instrSession)) < 0) {
                    errorQuit("rsupl_confInstrDisplay", status);
                }
                if ((status = rsupl_measSingleCh (instrSession, VI_FALSE, 0, &value)) < 0) {
                    errorQuit("rsupl_confInstrDisplay", status);
                }
                // display measured data
                sprintf(strBuf, "%f", value);
                InsertListItem (panelHdl, PANEL_LISTBOX, -1, strBuf, i);
            }
            
            if (audioMonitor) {
                // switch off audio monitor
                if ((status = rsupl_confInstrAudioMonitor (instrSession, VI_FALSE, 0, VI_FALSE,
                                                           0.0, 30.0)) < 0) {
                    errorQuit("rsupl_confInstrDisplay", status);
                }
            }
            
            // remove configuration dialog
            RemovePopup (1);
            break;
        }
    return 0;
}

int CVICALLBACK quitCall (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:
            QuitUserInterface (0);
            break;
        }
    return 0;
}

